////////////////////////////////////////////////////////////////////////////////////////////////////
// NoesisGUI - http://www.noesisengine.com
// Copyright (c) 2013 Noesis Technologies S.L. All Rights Reserved.
////////////////////////////////////////////////////////////////////////////////////////////////////


#include <NsCore/Noesis.h>
#include <NsCore/TypeId.h>
#include <NsCore/ReflectionImplement.h>
#include <NsCore/RegisterComponent.h>
#include <NsApp/EntryPoint.h>
#include <NsApp/Application.h>
#include <NsApp/EmbeddedXamlProvider.h>
#include <NsApp/EmbeddedFontProvider.h>
#include <NsApp/ApplicationLauncher.h>
#include <NsApp/Window.h>
#include <NsGui/IntegrationAPI.h>
#include <NsGui/SolidColorBrush.h>
#include <NsGui/ObservableCollection.h>
#include <NsDrawing/Color.h>

#include <NsGui/Button.h>

#include "App.xaml.bin.h"
#include "MainWindow.xaml.bin.h"

#include <string>
#include <assert.h>

using namespace Noesis;
using namespace NoesisApp;

namespace UserControl
{

////////////////////////////////////////////////////////////////////////////////////////////////////
class App final: public Application
{
    NS_IMPLEMENT_INLINE_REFLECTION(App, Application)
    {
        NsMeta<TypeId>("UserControl.App");
    }
};

class MyComponent : public BaseComponent {
public:
	NsString m_name;
	MyComponent() = default;
	MyComponent(const std::string& name) : m_name(name.c_str()) {}
	~MyComponent() {
		;
	}
private:
	NS_IMPLEMENT_INLINE_REFLECTION(MyComponent, BaseComponent) {
		NsMeta<TypeId>("MyComponent");
		NsProp("Name", &MyComponent::m_name);
	}
};

////////////////////////////////////////////////////////////////////////////////////////////////////
class MainWindow final: public Window
{
public:
    MainWindow()
    {
        InitializeComponent();
    }
private:
    void InitializeComponent()
    {
		m_first_ctx = *new Noesis::ObservableCollection<MyComponent>();
		m_second_ctx = *new Noesis::ObservableCollection<MyComponent>();

		for (int i = 0; i < 5; ++i) {
			Noesis::Ptr<MyComponent> new_item1 = *new MyComponent("blah " + std::to_string(i));
			m_first_ctx->Add(new_item1.GetPtr());

			Noesis::Ptr<MyComponent> new_item2 = *new MyComponent("qwer " + std::to_string(i));
			m_second_ctx->Add(new_item2.GetPtr());
		}

        Noesis::GUI::LoadComponent(this, "MainWindow.xaml");
    }
	bool ConnectEvent(BaseComponent* source, const char* event, const char* handler) {
		NS_CONNECT_EVENT(Noesis::Button, Click, OnButtonClick);
		return false;
	}

	void OnButtonClick(BaseComponent* sender, const RoutedEventArgs& args) {
		m_first_ctx->Clear();
		m_second_ctx->Clear();
	}

private:
	Noesis::Ptr<Noesis::ObservableCollection<MyComponent> > m_first_ctx;
	Noesis::Ptr<Noesis::ObservableCollection<MyComponent> > m_second_ctx;
	
    NS_IMPLEMENT_INLINE_REFLECTION(MainWindow, Window)
    {
        NsMeta<TypeId>("UserControl.MainWindow");
		NsProp("FirstTabCtx", &MainWindow::m_first_ctx);
		NsProp("SecondTabCtx", &MainWindow::m_second_ctx);
    }
};

}

////////////////////////////////////////////////////////////////////////////////////////////////////
class AppLauncher final: public ApplicationLauncher
{
private:
    void RegisterComponents() const override
    {
        NsRegisterComponent<::UserControl::App>();
		NsRegisterComponent<::UserControl::MyComponent>();
		NsRegisterComponent<::UserControl::MainWindow>();
    }

    Ptr<XamlProvider> GetXamlProvider() const override
    {
        EmbeddedXaml xamls[] = 
        {
            { "App.xaml", App_xaml, sizeof(App_xaml) },
            { "MainWindow.xaml", MainWindow_xaml, sizeof(MainWindow_xaml) },
        };

        return *new EmbeddedXamlProvider(xamls, NS_COUNTOF(xamls));
    }

};

////////////////////////////////////////////////////////////////////////////////////////////////////
int NsMain(int argc, char **argv)
{
    AppLauncher launcher;
    launcher.SetArguments(argc, argv);
    launcher.SetApplicationFile("App.xaml");
    return launcher.Run();
}
